"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("./index.scss");
var React = require("react");
var cx = require("classnames");
var recompose_1 = require("recompose");
var aerial_common2_1 = require("aerial-common2");
var state_1 = require("front-end/state");
var actions_1 = require("front-end/actions");
var getBreadcrumbNodes = aerial_common2_1.weakMemo(function (workspace, browser) {
    if (workspace.selectionRefs.length === 0) {
        return [];
    }
    var _a = workspace.selectionRefs[workspace.selectionRefs.length - 1], type = _a[0], $id = _a[1];
    if (type !== state_1.SYNTHETIC_ELEMENT) {
        return [];
    }
    var node = state_1.getSyntheticNodeById(browser, $id);
    // not ready yet
    if (!node) {
        return [];
    }
    var ancestors = state_1.getSyntheticNodeAncestors(node, state_1.getSyntheticNodeWindow(browser, node.$id)).filter(function (node) { return node.$type === state_1.SYNTHETIC_ELEMENT; }).reverse();
    return ancestors.concat([node]);
});
var BreadcrumbBase = function (_a) {
    var element = _a.element, onClick = _a.onClick, selected = _a.selected, onMouseEnter = _a.onMouseEnter, onMouseLeave = _a.onMouseLeave;
    return React.createElement("div", { className: cx("breadcrumb fill-text", { selected: selected }), onClick: onClick, onMouseEnter: onMouseEnter, onMouseLeave: onMouseLeave },
        state_1.getSyntheticElementLabel(element),
        selected ? null : React.createElement("span", { className: "arrow" },
            React.createElement("i", { className: "ion-ios-arrow-right" })));
};
var enhanceBreadcrumb = recompose_1.compose(recompose_1.pure, recompose_1.withHandlers({
    onClick: function (_a) {
        var dispatch = _a.dispatch, element = _a.element, windowId = _a.windowId;
        return function () {
            dispatch(actions_1.breadcrumbItemClicked(element.$id, windowId));
        };
    },
    onMouseEnter: function (_a) {
        var dispatch = _a.dispatch, element = _a.element, windowId = _a.windowId;
        return function () {
            dispatch(actions_1.breadcrumbItemMouseEnter(element.$id, windowId));
        };
    },
    onMouseLeave: function (_a) {
        var dispatch = _a.dispatch, element = _a.element, windowId = _a.windowId;
        return function () {
            dispatch(actions_1.breadcrumbItemMouseEnter(element.$id, windowId));
        };
    }
}));
var Breadcrumb = enhanceBreadcrumb(BreadcrumbBase);
var BreadcrumbsBase = function (_a) {
    var workspace = _a.workspace, browser = _a.browser, dispatch = _a.dispatch;
    var breadcrumbNodes = getBreadcrumbNodes(workspace, browser);
    return React.createElement("div", { className: "m-html-breadcrumbs" }, breadcrumbNodes.map(function (node, i) {
        return React.createElement(Breadcrumb, { key: node.$id, dispatch: dispatch, element: node, windowId: state_1.getSyntheticNodeWindow(browser, node.$id).$id, selected: i === breadcrumbNodes.length - 1 });
    }));
};
var Breadcrumbs = BreadcrumbsBase;
exports.Breadcrumbs = Breadcrumbs;
//# sourceMappingURL=index.js.map