"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
require("./overlay.scss");
var cx = require("classnames");
var React = require("react");
var Hammer = require("react-hammerjs");
var lodash_1 = require("lodash");
var recompose_1 = require("recompose");
var aerial_browser_sandbox_1 = require("aerial-browser-sandbox");
var aerial_common2_1 = require("aerial-common2");
var actions_1 = require("front-end/actions");
var NodeOverlayBase = function (_a) {
    var windowId = _a.windowId, zoom = _a.zoom, bounds = _a.bounds, node = _a.node, dispatch = _a.dispatch, hovering = _a.hovering;
    if (!bounds) {
        return null;
    }
    var borderWidth = 2 / zoom;
    var style = {
        left: bounds.left,
        top: bounds.top,
        width: bounds.right - bounds.left,
        height: bounds.bottom - bounds.top,
        boxShadow: "inset 0 0 0 " + borderWidth + "px #00B5FF"
    };
    return React.createElement("div", { className: cx("visual-tools-node-overlay", { hovering: hovering }), style: style });
};
var NodeOverlay = recompose_1.pure(NodeOverlayBase);
var WindowOverlayToolsBase = function (_a) {
    var dispatch = _a.dispatch, window = _a.window, hoveringNodes = _a.hoveringNodes, zoom = _a.zoom, onPanStart = _a.onPanStart, onPan = _a.onPan, onPanEnd = _a.onPanEnd;
    if (!window.allComputedBounds) {
        return null;
    }
    var style = {
        position: "absolute",
        left: window.bounds.left,
        top: window.bounds.top,
        width: window.bounds.right - window.bounds.left,
        height: window.bounds.bottom - window.bounds.top
    };
    return React.createElement("div", { style: style },
        React.createElement(Hammer, { onPanStart: onPanStart, onPan: onPan, onPanEnd: onPanEnd, direction: "DIRECTION_ALL" },
            React.createElement("div", { style: { width: "100%", height: "100%", position: "absolute" }, onDoubleClick: aerial_common2_1.wrapEventToDispatch(dispatch, actions_1.stageToolOverlayMouseDoubleClicked.bind(_this, window.$id)) }, hoveringNodes.map(function (node) { return React.createElement(NodeOverlay, { windowId: window.$id, zoom: zoom, key: node.$id, node: node, bounds: window.allComputedBounds[node.$id], dispatch: dispatch, hovering: true }); }))));
};
var enhanceWindowOverlayTools = recompose_1.compose(recompose_1.pure, recompose_1.withHandlers({
    onPanStart: function (_a) {
        var dispatch = _a.dispatch, window = _a.window;
        return function (event) {
            dispatch(actions_1.stageToolOverlayMousePanStart(window.$id));
        };
    },
    onPan: function (_a) {
        var dispatch = _a.dispatch, window = _a.window;
        return function (event) {
            dispatch(actions_1.stageToolOverlayMousePanning(window.$id, { left: event.center.x, top: event.center.y }, event.deltaY, event.velocityY));
        };
    },
    onPanEnd: function (_a) {
        var dispatch = _a.dispatch, window = _a.window;
        return function (event) {
            event.preventDefault();
            setImmediate(function () {
                dispatch(actions_1.stageToolOverlayMousePanEnd(window.$id));
            });
        };
    }
}));
var WindowOverlayTools = enhanceWindowOverlayTools(WindowOverlayToolsBase);
var getSyntheticNodes = aerial_common2_1.weakMemo(function (refs, allNodes) {
    return refs.map(function (_a) {
        var type = _a[0], id = _a[1];
        return allNodes[id];
    }).filter(function (id) { return !!id; });
});
var getHoveringSyntheticNodes = aerial_common2_1.weakMemo(function (workspace, window) {
    var allNodes = aerial_browser_sandbox_1.getSyntheticWindowChildStructs(window);
    return lodash_1.difference(getSyntheticNodes(workspace.hoveringRefs, allNodes), getSyntheticNodes(workspace.selectionRefs, allNodes));
});
exports.NodeOverlaysToolBase = function (_a) {
    var workspace = _a.workspace, browser = _a.browser, dispatch = _a.dispatch, zoom = _a.zoom;
    return React.createElement("div", { className: "visual-tools-layer-component" }, browser.windows.map(function (window) {
        return React.createElement(WindowOverlayTools, { key: window.$id, hoveringNodes: getHoveringSyntheticNodes(workspace, window), window: window, dispatch: dispatch, zoom: zoom });
    }));
};
exports.NodeOverlaysTool = recompose_1.pure(exports.NodeOverlaysToolBase);
//# sourceMappingURL=overlay.js.map