"use strict";
// TODO - measurements
Object.defineProperty(exports, "__esModule", { value: true });
var CSSDeclarationExpressionType;
(function (CSSDeclarationExpressionType) {
    CSSDeclarationExpressionType[CSSDeclarationExpressionType["COLOR"] = 0] = "COLOR";
    CSSDeclarationExpressionType[CSSDeclarationExpressionType["VAR"] = 1] = "VAR";
    CSSDeclarationExpressionType[CSSDeclarationExpressionType["CALL"] = 2] = "CALL";
    CSSDeclarationExpressionType[CSSDeclarationExpressionType["KEYWORD"] = 3] = "KEYWORD";
    CSSDeclarationExpressionType[CSSDeclarationExpressionType["NUMBER"] = 4] = "NUMBER";
})(CSSDeclarationExpressionType = exports.CSSDeclarationExpressionType || (exports.CSSDeclarationExpressionType = {}));
;
var CSSDeclarationTokenType;
(function (CSSDeclarationTokenType) {
    CSSDeclarationTokenType[CSSDeclarationTokenType["WORD"] = 0] = "WORD";
    CSSDeclarationTokenType[CSSDeclarationTokenType["NUMBER"] = 1] = "NUMBER";
    CSSDeclarationTokenType[CSSDeclarationTokenType["CHAR"] = 2] = "CHAR";
    CSSDeclarationTokenType[CSSDeclarationTokenType["WS"] = 3] = "WS";
})(CSSDeclarationTokenType = exports.CSSDeclarationTokenType || (exports.CSSDeclarationTokenType = {}));
;
var token = function (type, value, position) { return ({
    type: type,
    value: value,
    position: position
}); };
var tokenize = function (source) {
    var position = 0;
    var n = source.length;
    var tokens = [];
    while (position < n) {
        var char = source.charAt(position);
        if (/\d/.test(char) || ((char === "-" || char === ".") && /\d+/.test(source.charAt(position + 1)))) {
            var value = source.substr(position).match(/^(-?[\d\.]+)/)[1];
            tokens.push(token(CSSDeclarationTokenType.NUMBER, value, position));
            position += value.length - 1;
        }
        else if (/\w+/.test(char) || (char === "-" && /[\w-]+/.test(source.charAt(position + 1)))) {
            var value = source.substr(position).match(/^([\w-]+)/)[1];
            tokens.push(token(CSSDeclarationTokenType.WORD, value, position));
            position += value.length - 1;
        }
        else if (/[\s\t]/.test(char)) {
            var value = source.substr(position).match(/^([\s\r\n\t]+)/)[1];
            tokens.push(token(CSSDeclarationTokenType.WS, char, position));
            position += value.length - 1;
        }
        else {
            tokens.push(token(CSSDeclarationTokenType.CHAR, char, position));
        }
        position++;
    }
    return tokens;
};
var _memos = {};
setInterval(function () {
    _memos = {};
}, 1000 * 60);
var getTokenType = function (tokens, position) {
    return tokens[position] && tokens[position].type;
};
var getTokenValue = function (tokens, position) {
    return tokens[position] && tokens[position].value;
};
var getTokenPosition = function (tokens, position) {
    return tokens[position] ? tokens[position].position : tokens[tokens.length - 1].position + tokens[tokens.length - 1].value.length;
};
exports.parseDeclarationValue = function (value) {
    if (_memos[value]) {
        return _memos[value];
    }
    return _memos[value] = getExpression(tokenize(value).filter(function (token) { return token.type !== CSSDeclarationTokenType.WS; }), { position: 0 });
};
var getExpression = function (tokens, cursor) {
    var currentType = getTokenType(tokens, cursor.position);
    if (currentType === CSSDeclarationTokenType.WORD) {
        if (getTokenValue(tokens, cursor.position + 1) === "(") {
            return getCall(tokens, cursor);
        }
        return getKeyword(tokens, cursor);
    }
    else if (currentType === CSSDeclarationTokenType.NUMBER) {
        return getNumber(tokens, cursor);
    }
    throw new Error("unknown token " + getTokenValue(tokens, cursor.position));
};
var getCall = function (tokens, cursor) {
    var startToken = tokens[cursor.position];
    cursor.position++; // eat word
    var params = getParams(tokens, cursor);
    return {
        name: startToken.value,
        type: CSSDeclarationExpressionType.CALL,
        params: params,
        location: { start: startToken.position, end: getTokenPosition(tokens, cursor.position) }
    };
};
var getKeyword = function (tokens, cursor) {
    cursor.position++; // eat word
    return {
        name: getTokenValue(tokens, cursor.position - 1),
        type: CSSDeclarationExpressionType.KEYWORD,
        location: { start: tokens[cursor.position - 1].position, end: getTokenPosition(tokens, cursor.position) }
    };
};
var getNumber = function (tokens, cursor) {
    cursor.position++;
    return {
        value: getTokenValue(tokens, cursor.position - 1),
        type: CSSDeclarationExpressionType.NUMBER,
        location: { start: tokens[cursor.position - 1].position, end: getTokenPosition(tokens, cursor.position) }
    };
};
var getParams = function (tokens, cursor) {
    var params = [];
    cursor.position++; // eat (
    while (1) {
        var current = getTokenValue(tokens, cursor.position);
        if (current === ",") {
            cursor.position++;
            continue;
        }
        if (current === ")") {
            break;
        }
        params.push(getExpression(tokens, cursor));
    }
    cursor.position++; // eat )
    return params;
};
exports.stringifyCSSExpression = function (expr) {
    if (expr.type === CSSDeclarationExpressionType.CALL) {
        var call = expr;
        return call.name + "(" + call.params.map(exports.stringifyCSSExpression).join(", ") + ")";
    }
    else if (expr.type === CSSDeclarationExpressionType.NUMBER || expr.type === CSSDeclarationExpressionType.COLOR) {
        return expr.value;
    }
    else if (expr.type === CSSDeclarationExpressionType.KEYWORD) {
        return expr.name;
    }
};
//# sourceMappingURL=css.js.map